// ¿Qué es una URI, URL y URN?
// La respuesta por ejemplo aquí: http://otroblogmas.com/diferencias-url-uri-urn/
// Info aquí del código: http://cordova.apache.org/docs/es/3.1.0/cordova_camera_camera.md.html

// Funcion para debugear este script
function alerta(mensaje) {

  if(config.debug)
    alert(mensaje);
}


var pictureSource;   // origen de la imagen
var destinationType; // establece el formato del valor devuelto

var tuimagen; //id de la imagen a colocar en el html
var mostrarimagen; //id del div a poner como visible o invisible en el html

var imageDataGlobal; // variable que contienen la imagen en base64 a subir

// Espera a que Cordova esté listo
document.addEventListener("deviceready", onDeviceReady, false);

// Cordova esta preparado para funcionar
function onDeviceReady() {
  alerta("onDeviceReady");

  pictureSource = navigator.camera.PictureSourceType;
  destinationType = navigator.camera.DestinationType;
}

// Funcion para colocarle a un boton
function capturePhoto() {
  alerta("capturePhoto");

  // Captura la foto usando la camara del dispositivo y recupera la imagen como cadena codificada en base64
  navigator.camera.getPicture(onPhotoDataSuccess, onFail, {quality: 50, destinationType: destinationType.DATA_URL});
}

// Funcion para colocarle a un boton
function getPhoto(source) {
  alerta("getPhoto");

  // Recuperar la ubicacion de una imagen de origen especificado
  // navigator.camera.getPicture(onPhotoURISuccess, onFail, { quality: 50, destinationType: destinationType.FILE_URI, sourceType: source });
  // Recuperar la ubicacion de una imagen de origen especificado en base64
  navigator.camera.getPicture(onPhotoDataSuccess, onFail, {quality: 50, destinationType: destinationType.DATA_URL, sourceType: source});

}

// Esta funcion es llamada cuando una foto es capturada de la cámara o de la galería correctamente. La información que le llega está codificada en base64
function onPhotoDataSuccess(imageData) {
  alerta("onPhotoDataSucces");

  // Elimina el comentario para ver los datos de imagen codificados en base64
  // console.log(imageData);
  imageDataGlobal = imageData;

  // Borro si existe alguna imagen anterior  
  // Capturo la id del html para colocar la imagen
  tuimagen = document.getElementById('tuimagen');

  //Cambiando el estilo CSS de la imagen para que sea visible como 'block'. Poner más estilos si se quiere.
  // tuimagen.style.display = 'block';
  mostrarimagen = document.getElementById('mostrarimagen');
  mostrarimagen.style.display = 'block';

  // Colocamos la imagen obtenida en la id elegida con tuimagen y le decodificamos la informacion que llega en base64
  tuimagen.src = "data:image/jpeg;base64," + imageData;
}

// Funcion llamada si algo sale mal
function onFail(message) {
  // alerta('Error: ' + message); // Por defecto cordova envía el message ="option cancelled"
  alert('Opción cancelada.');
}


// Funcion para enviar la imagen al Panel del Control
function sendPhotoToServer() {
  alerta("sendPhotoToServer");

  jQuery.ajax ({
    url: "http://seiscocos.com:3000/api/" + config.backoffice_user + "/FotosUsuarios/new",
    type: "POST",
    async: true,
    data: JSON.stringify({url:imageDataGlobal}),
    dataType: "json",
    contentType: "application/json; charset=utf-8",
    success: function() {
      alert("Enviado correctamente. Una vez validada su foto aparecerá en la sección 'Fotos de usuarios'.");

      // Borro la URI de la imagen del html
      tuimagen = document.getElementById('tuimagen');
      tuimagen.src = '';

      // Oculto la barra de progreso
      var barra = document.getElementById('barra');
      barra.style.display = 'none';

      // Quita la vista de la imagen del html
      mostrarimagen = document.getElementById('mostrarimagen');
      mostrarimagen.style.display = 'none';

      // Borro la informacion de la imagen capturada
      imageData = null;
    },
    xhr: function() {
      var xhr = $.ajaxSettings.xhr();
      xhr.upload.onprogress = function(evt) {
        //Estilos de la barra de progreso
        console.log( (evt.loaded/evt.total)*100 );
        
        var barra = document.getElementById('barra');
        barra.style.display = 'block';
        
        var progreso = document.getElementById('progreso');
        progreso.style.width = ((evt.loaded/evt.total)*100) + "%";
        progreso.style.height = "100%";
        progreso.style.backgroundColor = "rgba(204, 21, 18, 1)";
        
        var num_progreso = document.getElementById('num_progreso');
        if(((evt.loaded/evt.total)*100) == 100) {
          num_progreso.innerHTML = "Esperando respuesta del servidor.";          
        }
        else {
          num_progreso.innerHTML = ((evt.loaded/evt.total)*100).toString().substr(0,5) + " %";
        }
      }
      return xhr;
    }
  });

  // Otra forma Javascript puro
  // xhr = new XMLHttpRequest();
  // xhr.open("POST", "http://seiscocos.com:3000/api/" + config.backoffice_user + "/FotosUsuarios/new", true);
  // xhr.setRequestHeader("Content-Type", "application/json");
  // xhr.onreadystatechange = function(){
  //   if(xhr.readyState == 4){

  //     alerta(xhr.responseText);
  //   }
  // }
  // xhr.send({url:imageData});

  // http://www.dave-bond.com/blog/2010/01/JQuery-ajax-progress-HMTL5/
  // https://github.com/englercj/jquery-ajax-progress
  // xhr: function() {
  //   var xhr = $.ajaxSettings.xhr();
  //   xhr.upload.onprogress = function(evt){
   
  //     //Estilos de la barra de progreso
  //     console.log( (evt.loaded/evt.total)*100 );
      
  //     var barra = document.getElementById('barra');
  //     barra.style.display = 'block';
      
  //     var progreso = document.getElementById('progreso');
  //     progreso.style.width = ((evt.loaded/evt.total)*100) + "%";
  //     progreso.style.height = "100%";
  //   }
  //   return xhr;
  // }
}

// Esta variable y función es llamada en 'templates.js' para el Handerbals
// de elección del 'albumtype' = 'Galería' o 'Fotos' de 'album.html'
var abt;
function albumtype(tipo) {
  alerta('%c albumtype = ' + tipo, 'color:red;');
  abt = tipo;
}