/**
 * External data views
 * @namespace
 */
var ext = {
    /**
     * Opens the given URL in a new panel
     * @param {string} title New panel header title
     * @param {string} url URL to open
     * @param {boolean} [cached=false] Cache the result for a week, then check updates in the background
     * @public
     */
	open: function(){
		var url;
		var changeThread;
		var name;
		if(arguments.length == 2){
			name = arguments[0];
			url = arguments[1];
			changeThread = false;
		}else if(arguments.length == 3){
			name = arguments[0];
			url = arguments[1];
			changeThread = arguments[2];
		}else{
			out.error("Max arguments are 3");
			return false;
		}

		var urlmd5 = hex_md5(url);
		var panel = nav.$(urlmd5);
		if(panel){
			panel.open(changeThread);
			return;
		}
		var panel;
		if(changeThread){
			panel = nav.newPanel(urlmd5,name,false);
		}else{
			panel = nav.newPanel(urlmd5,name,true);
		}
		var refreshButton = document.createElement("i");
		refreshButton.setAttribute("action","refresh");
		refreshButton.setAttribute("panelname",urlmd5);
		refreshButton.className = "fa fa-refresh fa-spin";
		panel.querySelector("header").appendChild(refreshButton);
		panel.loadingOn();
		panel.errorOff();
		panel.setAttribute("url", url);
		panel.open(changeThread);
		panel.setAttribute("type","external");

		xhr = new XMLHttpRequest();
		xhr.onreadystatechange = function(){
			if(xhr.readyState == 4 && xhr.response != "") {
				var jR = JSON.parse(xhr.responseText);
				if(jR.statusCode == 1){
					panel.setAttribute("datatype",jR.type);
					if(jR.newURL){
						panel.setAttribute("newurl",jR.newURL);
					}
					if(jR.nextURL){
						panel.setAttribute("nexturl",jR.nextURL);
					}
					panel.loadingOff();
					templates.apply(jR.type, jR, function(res){
						panel.html(res);
					});
				}
			}else if(xhr.readyState == 4){
				panel.errorOn();
				panel.loadingOff();
			}
		}
		xhr.open("GET",url,true);
		xhr.send();
	},
	refresh: function(pname){
		var panel = nav.$(pname);
		if(panel){
			var insert = false;
			if(panel.isLoading()){ return; }
			if(panel.getAttribute("newurl")){
				var url = panel.getAttribute("newurl");
				insert = true;
			}else{
				var url = panel.getAttribute("url");
			}
			panel.loadingOn();
			panel.errorOff();
			xhr = new XMLHttpRequest();
			xhr.onreadystatechange = function(){
				if (xhr.readyState == 4 && xhr.response != "") {
					var jR = JSON.parse(xhr.responseText);
					if(jR.statusCode == 1){
						panel.setAttribute("datatype",jR.type);
					}
					if(jR.statusCode == 1 && jR.data.length > 0){
						if(jR.newurl){
							panel.setAttribute("newurl",jR.newURL);
						}
						if(jR.nexturl){
							panel.setAttribute("nexturl",jR.nextURL);
						}
						templates.apply(jR.type, jR, function(res){
							if(insert){
								var resCont = document.createElement("div");
								resCont.innerHTML = res;
								var nodelist = new Array();
								utils.eachChild(resCont, function(d){
									nodelist.push(d);
								});
								nodelist = nodelist.reverse();
								for(i=0;i<nodelist.length;i++){
									panel.prepend(nodelist[i]);
								}
								out.log("Refresh: Loading new data");
							}else{
								panel.html(res);
								out.log("Refresh: No new data");
							}
							panel.loadingOff();
						});
					}else{
						panel.loadingOff();
					}
				}else if(xhr.readyState == 4){
					panel.errorOn();
					panel.loadingOff();
				}
			}
			xhr.open("GET",url,true);
			xhr.send();
		}
	}
}