/**
 * Maps Management
 * @namespace
 */
var maps = {
	list: [],
	internalIteration: function(panels){
		var panel = panels.pop();
		if(panel.getAttribute("maploaded") == "true"){
			maps.internalIteration(panels);
			return;
		}
		panel.setAttribute("maploaded","true");
		panel.setAttribute("loading","true");
		var newMap = new google.maps.Map(panel.querySelector("content"), {
			zoom: 8,
			mapTypeId: google.maps.MapTypeId.ROADMAP
		});

		var newDr = new google.maps.DirectionsRenderer();
		newDr.setMap(newMap);

		maps.locAlias(panel.getAttribute("from"),panel.getAttribute("to"), function(locsR){
			var newDs = new google.maps.DirectionsService();
			newDs.route({
				origin: locsR[0],
				destination: locsR[1],
				travelMode: google.maps.DirectionsTravelMode.DRIVING
			}, function(res,status){
				if(status == google.maps.DirectionsStatus.OK){
					newDr.setDirections(res);
					maps.list.push({map:newMap,dr:newDr,ds:newDs});
					var mapid = maps.list.length-1;
					panel.setAttribute("mapid",mapid);
					panel.setAttribute("loading","false");
					panel.querySelector("*[action='refresh']").setAttribute("mapid",mapid);
				}
				if(panels.length > 0){
					maps.internalIteration(panels);
				}
			});
		})
	},
	/**
     * Initialize maps detection
     * @public
     */
	init: function(){
		if(arguments.length == 1){
			var panels = document.querySelectorAll("panel[type='map'][name='" + arguments[0] + "']:not([maploaded='true'])");
		}else{
			var panels = document.querySelectorAll("panel[type='map']:not([maploaded='true'])");
		}
		var array = [];
		for(var i = 0;i<panels.length;i++){
			array.push(panels[i]);
		}
		if(array.length > 0){
			out.log("maps.init: Started " + array.length + " maps load iteration");
			maps.internalIteration(array);
		}else{
			out.log("maps.init: 0 maps to load");
		}
	},
	/**
     * Refresh the selected map
     * @param {integer} Map ID
     * @public
     */
	refresh: function(id){
		id = parseInt(id);
		var panel = document.querySelector("panel[type='map'][mapid='" + id + "']");
		panel.setAttribute("loading","true");
		maps.locAlias(panel.getAttribute("from"),panel.getAttribute("to"), function(locsR){
			maps.list[id].map.setZoom(8);
			maps.list[id].ds.route({
				origin: locsR[0],
				destination: locsR[1],
				travelMode: google.maps.DirectionsTravelMode.DRIVING
			}, function(res,status){
				if (status == google.maps.DirectionsStatus.OK) {
					maps.list[id].dr.setDirections(res);
				}
				panel.setAttribute("loading","false");
			});
		});
	},
	locAlias: function(){
		if(arguments.length <= 1){
			out.error("maps.locAlias needs at least a location and a callback");
			return false;
		}
		var list = [];
		for(i=0;i<arguments.length;i++){
			list.push(arguments[i]);
		}
		maps.locAliasIteration(list.slice(0,list.length-1), list.slice(list.length-1)[0]);
	},
	locAliasIteration: function(locationList, callback){
		var next = locationList.pop();
		if(!next) {
			callback(maps.locAliasList.reverse());
			return;
		}
		if(next == "here"){
			out.log("MAPS: Getting geolocation");
			navigator.geolocation.getCurrentPosition(function(res){
				var resloc = res.coords.latitude + ", " + res.coords.longitude;
				maps.locAliasList.push(resloc);
				out.log(resloc);
				maps.locAliasIteration(locationList, callback);
			}, function(){
				maps.locAliasList.push("");
				maps.locAliasIteration(locationList, callback);
			});
		}else{
			maps.locAliasList.push(next);
			maps.locAliasIteration(locationList, callback);
		}
	},
	locAliasList: [],
	openNative: function(destiny){
		if(document.body.getAttribute("platform") == "android"){
			to = destiny.replace(/ /g,"+");
			window.open("http://maps.google.com/maps?daddr=" + to, "_system");
		}else{
			to = destiny.replace(/ /g,"+");
			var url = "http://maps.apple.com/?daddr=" + to;
			out.log("Open Maps App: " + url);
			var ref = window.open(url, "_system");
		}
	}
}