/**
 * Navigation
 * @namespace
 */
var nav = {
	init: function(){
		nav.history.init();
		templates.apply("_connection_problem", {}, function(res){
			nav.connection_problem = res;
		});
	},
	connection_problem: "",
	history: {
		threads: {},
		getActiveThread: function(){
			if(document.querySelector("navbar")){
				return nav.history.threads[document.querySelector("navbar tab[status='active']").getAttribute("name")];
			}else{
				return nav.history.threads["main"];
			}
		},
		getActiveTabFirstPanel: function(){
			if(document.querySelector("navbar")){
				var firstpanel = nav.alias.get(document.querySelector("navbar tab[status='active']").getAttribute("firstpanel"));
				if(utils.isURL(firstpanel)){
					firstpanel = hex_md5(firstpanel);
				}
				return firstpanel;
			}else{
				return "main";
			}
		},
		init: function(){
			if(document.querySelector("navbar")){
				var tabs = document.querySelectorAll("navbar tab");
				for(var i=0;i<tabs.length;i++){
					nav.history.threads[tabs[i].getAttribute("name")] = Array();
				}
			}else{
				nav.history.threads["main"] = Array();
			}
			
		},
		push: function(pname){
			var activeThread = nav.history.getActiveThread();
			if(utils.isURL(pname)){
				pname = hex_md5(pname);
			}
			activeThread.push(pname);
			if(activeThread.length == 1){
				app.bindBack();
			}
		},
		pop: function(){
			var activeThread = nav.history.getActiveThread();
			if(activeThread.length == 0){
				return nav.history.getActiveTabFirstPanel();
			}else{
				var res = activeThread.pop();
				if(activeThread.length == 0){
					app.unBindBack();
				}
				return res;
			}
		}
	},
	/**
	 * Returns the selected panel
	 * @param {string} pname Panel's name
	 * @public
	 */
	$: function(pname){
		var panel = document.querySelectorAll("panel[name='" + pname + "']").item(0);
		if(panel){
			panel.name = panel.getAttribute("name");
			panel.status = panel.getAttribute("status");
		}else{
			return false;
		}
		panel.prepend = function(element){
			var content = this.querySelector("content");
			content.insertBefore(element,content.firstChild);
		}
		panel.html = function(){
			if(arguments.length == 0){
				return this.querySelector("content").innerHTML;
			}else if(arguments.length == 1){
				this.querySelector("content").innerHTML = arguments[0];
			}
		}
		panel.text = function(){
			if(arguments.length == 0){
				return this.querySelector("content").innerText;
			}else if(arguments.length == 1){
				this.querySelector("content").innerText = arguments[0];
			}else{
				out.error("panel.text: Too much arguments, only accepted 0 or 1");
			}
		}
		panel.content = function(){
			return this.querySelector("content");
		}
		panel.open = function(){
			if(arguments.length == 0){
				nav.goTo(this.name);
			}else if(arguments.length == 1){
				if(arguments[0]){
					nav.goTo(this.name, arguments[0]);
				}else{
					nav.goTo(this.name);
				}
			}
			
		}
		panel.loadingOn = function(){
			this.setAttribute("loading","true");
			var ajax = document.createElement("ajax");
			this.prepend(ajax);
		}
		panel.loadingOff = function(){
			this.setAttribute("loading","false");
			var ajaxElement = this.querySelector("ajax");
			if(ajaxElement){
				try {
					ajaxElement.remove();
				}catch(err){
					// Internet Explorer Fallback
					ajaxElement.parentNode.removeChild(ajaxElement);
				}	
			}
		}
		panel.errorOn = function(){
			this.setAttribute("loaderror","true");
			var newdiv = document.createElement("div");
			newdiv.setAttribute("class","_error");
			newdiv.innerHTML = nav.connection_problem;
			this.prepend(newdiv);
		}
		panel.errorOff = function(){
			this.setAttribute("loaderror","false");
			var errorElement = this.querySelector("div._error");
			if(errorElement){
				try {
					errorElement.remove();
				}catch(err){
					// Internet Explorer Fallback
					errorElement.parentNode.removeChild(errorElement);
				}	
			}
		}
		panel.isLoading = function(){
			if(this.getAttribute("loading") == "true"){
				return true;
			}else{
				return false;
			}
		}
		return panel;
	},
	/**
	 * Navigates to selected panel
	 * @param {string} pname Panel's name
	 * @public
	 * @returns {boolean}
	 */
	goTo: function(){
		var pname, change;
		if(arguments.length == 1){
			pname = arguments[0];
			change = false;
		}else if(arguments.length == 2){
			pname = arguments[0];
			change = arguments[1];
		}else{
			out.error("goTo: Wrong arguments");
			return false;
		}
		if(pname && pname != ""){
			var nextPanel = nav.$(pname);
			if(nextPanel){
				if(nextPanel.getAttribute("type") == "map" && utils.isPhone()){
					var to = nextPanel.getAttribute("to");
					maps.openNative(to);
				}else{
					var actualPanel = document.querySelector("panel[status='shown']");
					if(change){
						var actualTab = document.querySelector("navbar tab[status='active']");
						this.history.push(actualPanel.getAttribute("name"));
						actualTab.setAttribute("status","");
						document.querySelector("navbar tab[name='" + change + "']").setAttribute("status","active");
						var nextPanelToNavName = nav.history.pop();
						var nextPanelToNav = nav.$(nextPanelToNavName);
						actualPanel.setAttribute("status","hidden");
						nextPanelToNav.setAttribute("status","shown");
					}else{
						actualPanel.setAttribute("status","hide");
						nextPanel.setAttribute("status","show");
						this.history.push(actualPanel.getAttribute("name"));
					}
				}
				return true;
			}else{
				out.error("goTo: Unknown panel");
				return false;
			}
		}else{
			out.error("goTo: Invalid or empty panel name");
			return true;
		}
	},
	/**
	 * Navigates back to the last panel in navigation history
	 * @public
	 */
	goBack: function(){
		var nextPanelName = this.history.pop();
		if(utils.isURL(nextPanelName)){
			nextPanelName = hex_md5(nextPanelName);
		}
		out.log("goBack: " + nextPanelName);
		if(nextPanelName){
			var actualPanel = document.querySelector("panel[status='shown']");
			var nextPanel   = nav.$(nextPanelName);
			if(actualPanel != nextPanel){
				actualPanel.setAttribute("status","hideBack");
				nextPanel.setAttribute("status","showBack");
			}
			return true;
		}else{
			return false;
		}
	},
	changeTo: function(tabname){

		var thisFirstPanel = nav.$(document.querySelector("tab[name='" + tabname + "']").getAttribute('firstpanel'));
		if(thisFirstPanel.getAttribute("type") == "map" && utils.isPhone()){
			var to = thisFirstPanel.getAttribute("to");
			maps.openNative(to);
			return;
		}

		var exists = nav.history.threads[tabname];
		if(exists){
			var actualTab = document.querySelector("navbar tab[status='active']");
			var actualPanel = document.querySelector("panel[status='shown']");
			var actualPanelName = actualPanel.getAttribute("name");
			nav.history.push(actualPanelName);
			actualTab.setAttribute("status","");
			document.querySelector("navbar tab[name='" + tabname + "']").setAttribute("status","active");
			var nextPanelName = nav.history.pop();
			var nextPanel = nav.$(nextPanelName);
			if(actualPanelName != nextPanelName){
				nextPanel.setAttribute("status","shown");
				actualPanel.setAttribute("status","hidden");
			}
		}
	},
	/**
	 * Creates a new panel
	 * @param {string} pname Panel's name
	 * @param {string} ptitle Panel's title
	 * @public
	 * @returns {Panel}
	 */
	newPanel: function(pname,ptitle){
		var pname, ptitle, includeback;
		if(arguments.length == 2){
			pname = arguments[0];
			ptitle = arguments[1];
			includeback = true;
		}else if(arguments.length == 3){
			pname = arguments[0];
			ptitle = arguments[1];
			includeback = arguments[2];
		}else{
			return false;
		}
		var panel = document.createElement("panel");
		var header = document.createElement("header");
		var name = document.createElement("name");
		var i = document.createElement("i");
		var content = document.createElement("content");
		//content.setAttribute("sc-element","myelement");
		//content.setAttribute("sc-limit","80");
		panel.setAttribute("name",pname);
		name.innerText = ptitle;
		if(includeback){
			i.setAttribute("class","fa fa-chevron-left");
			i.setAttribute("action","back");
			header.appendChild(i);
		}else{
			if(document.querySelector('body').getAttribute("platform") != "android"){
				i.setAttribute("class","fa");
				header.appendChild(i);
			}
		}
		header.appendChild(name);
		panel.appendChild(header);
		panel.appendChild(content);
		document.body.appendChild(panel);
		return nav.$(pname);
	},
	alias: {
		list: {},
		register: function(aliasname, action){
			if(typeof(aliasname) != "string") { return false; }
			var type = typeof(action);
			if(type == "string" || type == "function"){
				nav.alias.list[aliasname] = action;
				return true;
			}else{
				return false;
			}
		},
		get: function(aliasname){
			if(nav.alias.list[aliasname]){
				var action = nav.alias.list[aliasname];
				if(typeof(action) == "string"){
					return action;
				}else if(typeof(action) == "function"){
					return action();
				}else{
					return aliasname;
				}
				return nav.alias.list[aliasname];
			}else{
				return aliasname;
			}
		}
	}
}